#!/bin/sh

if [ $# -ne 1 ]; then
    echo "$0 ssh-keyfile"
    echo "   ssh-keyfile: ssh keys to add into the image"
    exit 1
fi

set -e
set -x

SSH_KEYS=$1; shift

MAJOR_VERSION=3.13
MINOR_VERSION=5
IMAGE=alpine-minirootfs-$MAJOR_VERSION.$MINOR_VERSION-x86_64.tar.gz

RESOLVCONF=/etc/resolv.conf

working_directory=$(pwd -P)
#rootfs_tmpdir=$(mktemp -d)
rootfs_tmpdir=alpine_${MAJOR_VERSION}-${MINOR_VERSION}-rootfs

initramfs="$working_directory/initramfs-alpine-${MAJOR_VERSION}.${MINOR_VERSION}"
kernel="$working_directory/kernel-alpine-${MAJOR_VERSION}.${MINOR_VERSION}"

mkdir -p ${rootfs_tmpdir}

rootfs_url="http://dl-cdn.alpinelinux.org/alpine/v$MAJOR_VERSION/releases/x86_64/$IMAGE"

run_root () {
    sudo chroot $rootfs_tmpdir /usr/bin/env \
        PATH=/bin:/sbin \
        /bin/sh -c "$*"
}

wget -c "$rootfs_url" -O "$IMAGE"

# keep right permissions, use sudo
sudo tar xf $IMAGE -C $rootfs_tmpdir

# Add SSH keys
run_root mkdir -p root/.ssh
sudo cp $SSH_KEYS $rootfs_tmpdir/root/.ssh/authorized_keys
run_root chown root:root /root/.ssh/authorized_keys
run_root chmod 0600 /root/.ssh/authorized_keys
run_root chmod 0700 /root/.ssh

# Import local resolv.conf.
sudo cp "$RESOLVCONF" $rootfs_tmpdir/etc/resolv.conf

# Make sure init is found by the kernel.
run_root ln -sf /sbin/init /init

run_root apk update
run_root apk add linux-lts openrc udev openssh rdnssd
# rdnssd
run_root rc-update add udev
run_root rc-update add udev-trigger
run_root rc-update add sshd
run_root rc-update add rdnssd
run_root rc-update add networking
run_root rc-update add hostname
run_root sed -i 's/root:!::0:::::/root:*::0:::::/' /etc/shadow

# Fix not yet updated initscript for rdnssd
sudo tee "$rootfs_tmpdir/etc/init.d/rdnssd" <<EOF
#!/sbin/openrc-run

supervisor=supervise-daemon
command=/usr/sbin/rdnssd
command_args="-H /etc/rdnssd/resolvconf"
command_args_foreground="-f"

start_pre() {
        checkpath -d -m 0755 -o nobody:nobody -q /run/rdnssd
}

EOF

sudo tee "$rootfs_tmpdir/etc/network/interfaces" <<EOF
auto lo
iface lo inet loopback

auto eth0
iface eth0 inet6 manual
  up /sbin/ip link set \$IFACE up

EOF

sudo tee "$rootfs_tmpdir/etc/hostname" <<EOF
alpine-unconfigured
EOF

# Cleanup resolv.conf
run_root rm -f /etc/resolv.conf

# Generate iniramfs image
(cd $rootfs_tmpdir; sudo find . | sudo cpio -H newc -o | gzip -9 > ${initramfs})
cp "$rootfs_tmpdir/boot/vmlinuz-lts" "${kernel}"

echo rm -rf "$rootfs_tmpdir"

echo "Use ${initramfs} and ${kernel} from $working_directory"!

exit 0
