#!/bin/sh
#
# Copyright 2020 -- Evilham <contact@evilham.com>
# This is BSD licensed as it's based on BSD-licensed code
#
#
# This builds all needed FreeBSD images for ungleich's Data Center Light
# When there are new releases, they should be updated here and the script
# should run.
# 11.4 is scheduled end of June 2020
# 12.2 is scheduled end of October 2020
#

SUPPORTED_RELEASES="11.3 12.1"

# This should run in a DCL VM with an OK amount of cores (4/8 minimum),
# 4G RAM, and storage of roughly 20G + 5G * #resulting_images.
#
# This is because there is the base system, a 'pristine chroot', and during the
# build there can be 2 copies of the resulting system written to the system.
# Since there are 4 combinations of images:
#   {STABLE,RELEASE} x {dualstack, IPv6ONLY}
#
# That means we'll need to assign about 40G storage to be on the safe side.

date=$(date -I)
for release in ${SUPPORTED_RELEASES}; do
    for build in dualstack ipv6only; do
        env RELEASE=${release} sh freebsd-build-opennebula-image-generic.sh --${build} \
            | tee "freebsd-${release}-${build}-${date}.log"
    done
done
