#!/bin/sh
# 2020-06-13, Nico Schottelius
# See https://ungleich.ch/u/products/viirb-ipv6-box/

if [ $# -ne 1 ]; then
    echo "$0 interface [address]"
    echo "    address: connect to this address, ignore the interface"
    exit 1
fi

set -x
vigir_ip=$1; shift

# openwrt
version=21.02.2
filename=openwrt-${version}-ramips-mt7621-zbtlink_zbt-wg3526-16m-squashfs-sysupgrade.bin
# don't care about other/old known_host entries
ssh-keygen  -R ${vigir_ip}

while ! ping -c1 ${vigir_ip}; do
    echo "Cannot ping $vigir_ip yet - waiting"
    sleep 1
done

cat ~/.ssh/id_rsa.pub | ssh root@${vigir_ip} "cat > /etc/dropbear/authorized_keys"

# Don't re-download if we already have it
wget -c http://downloads.openwrt.org/releases/${version}/targets/ramips/mt7621/${filename}

if echo $vigir_ip | grep -q :; then
    scp_ip="[$vigir_ip]"
else
    scp_ip="$vigir_ip"
fi

scp ${filename} root@${scp_ip}:/tmp
ssh root@${vigir_ip} "sysupgrade -n /tmp/*.bin"

# It still pings for some time - wait for the reboot to happen
echo "Waiting for vigir to really disappear"
sleep 15

wait=0
found=""

while [ $wait -lt 180 ]; do
    ping -c1 ${vigir_ip} >/dev/null

    if [ $? -eq 0 ]; then
        found=yes
        # wait for ssh to come up
        sleep 10
        break
    fi

    sleep 1
    wait=$((wait+1))
    done

if [ ! "$found" ]; then
    echo "Did not find updated vigir - debug / restart it"
    exit 1
fi

echo "vigir successfully updated to ${version}"
